% Nonlinear CSTR  model

% 7 states
% 3 inputs
% 3 disturbances

% Initial values:
% X_Glu = 0; % g/L
% Glu = 20; % g/L
% Man = 20; % g/L
% Xyl = 20; % g/L
% P = 0; % M
% V = 0.7 % L
% V_RiboGlu = 1 % L
%
% Constraints
% % For x = [X_RiboGlu Glu, Man, Xyl, P, V, V_RiboGlu]
% x_lower = [0 0 0 0 0 0.7 0.7 ]; 
% x_upper = [ inf,  inf, inf, inf, inf 1 1];
%
% % For u = [F_RiboGlu_in , F_SSL_in, F_out] % 
% u_lower = [0 0 0 ]; 
% u_upper = [1 1 1]; 
%
function ODEsys = ODEmodel()
%addpath ('/users/dinark/Downloads/casadi-osx-matlabR2015a-v3.5.1/')
import casadi.*
ODEsys = struct; %Empty struct

% States, declaring them symbolically
X_RiboXyl = SX.sym('X_RiboXyl'); %Biomass, g/L
Glu = SX.sym('Glu'); %Substrate, g/L
Man = SX.sym('Man'); %Substrate, g/L
Xyl = SX.sym('Xyl'); %Substrate, g/L
P= SX.sym('P'); % Riboflavin, M
V=SX.sym('V'); % Reactor volume, L
V_RiboXyl=SX.sym('V_RiboXyl');%RiboXyl reactor volume, L
%Filling up the ODEsys struct
ODEsys.x = [X_RiboXyl, Glu, Man, Xyl, P, V, V_RiboXyl]';
ODEsys.x_names = ['X_RiboXyl', 'Glu', 'Man', 'Xyl', 'P', 'V', 'V_RiboXyl']';

% Inputs - Normalized by reactor volume.
F_RiboXyl_in=SX.sym('F_RiboXyl_in'); %L/h
F_SSL_in=SX.sym('F_SSL_in'); %L/h - sugar stream
F_out= SX.sym('F_out');%L/h

ODEsys.u = [F_RiboXyl_in, F_SSL_in, F_out]';
ODEsys.u_names = [F_RiboXyl_in, F_SSL_in, F_out]';

% Uncertainties
% SSL feed in stream concentrations
SSL_Glu_in = SX.sym('SSL_Glu_in'); % g/L
SSL_Man_in = SX.sym('SSL_Man_in'); % g/L
SSL_Xyl_in = SX.sym('SSL_Xyl_in'); % g/L
ODEsys.d = [SSL_Glu_in,SSL_Man_in,SSL_Xyl_in]';
ODEsys.d0 = [36,110,45]'; % g/L SSL

%Substrate concentrations
%Assuming the S_in are zero, as they come from a completed batch with
%sugars consumed.
%Sugars from RiboXyl strain
RiboXyl_Glu_in=0;
RiboXyl_Man_in=0;
RiboXyl_Xyl_in=0;
%Biomass concentration
RiboXyl_X_RiboXyl_in=7;
%Riboflavin concentration
%Interested in the riboflavin produced in main reactor
RiboXyl_P_in=0;
SSL_P_in=0;


%% #####################################################
%                       Parameters
%              Obtained from Parameter_Esitmation_Strain

%% RiboXyl
mumax1_RiboXyl=0.0675;
mumax2_RiboXyl=0.0003;
mumax3_RiboXyl=0.1082;
Ks1_RiboXyl=0.0905;
Ks2_RiboXyl=653.7131;
Ks3_RiboXyl=2.0783;
Y_xs1_RiboXyl=-0.2396;
Y_xs2_RiboXyl=-0.0001;
Y_xs3_RiboXyl=-0.2876;
Y_px_RiboXyl=0.9751;

% #######################################
%               Constants
% #######################################

mu1_RiboXyl= mumax1_RiboXyl*Glu/(Ks1_RiboXyl+Glu);
mu2_RiboXyl = mumax2_RiboXyl*Man/(Ks2_RiboXyl+Man);
mu3_RiboXyl = mumax3_RiboXyl*Xyl/(Ks3_RiboXyl+Xyl);
rx1_RiboXyl = mu1_RiboXyl*X_RiboXyl;
rx2_RiboXyl = mu2_RiboXyl*X_RiboXyl;
rx3_RiboXyl = mu3_RiboXyl*X_RiboXyl;
rx_RiboXyl = rx1_RiboXyl + rx2_RiboXyl + rx3_RiboXyl;
rs1_RiboXyl = rx1_RiboXyl/Y_xs1_RiboXyl;
rs2_RiboXyl= rx2_RiboXyl/Y_xs2_RiboXyl;
rs3_RiboXyl = rx3_RiboXyl/Y_xs3_RiboXyl;
rp_RiboXyl=Y_px_RiboXyl*rx_RiboXyl;

%% Initial values of states

% Initial Values of states
ODEsys.x0 = [0, 20, 20, 20, 0, 0.7, 1]';
ODEsys.u0 = [0,0,0]';

% For x = [X_RiboGlu,  Glu, Man, Xyl, P, V, V_RiboGlu]
ODEsys.x_lower = [0, 0,0,0,0 0.7, 0.7]'; %
ODEsys.x_upper = [ inf, inf, inf, inf, inf, 1, 1]';

% For u = [F_RiboGlu_in, F_SSL_in, F_out] % 
  ODEsys.u_lower = [0, 0,0]'; 
  ODEsys.u_upper = [1, 1, 1]';
  %% Constant F_SSL_in of 0.1 L/h
%  ODEsys.u_lower = [0, 0.1,0]';
%  ODEsys.u_upper = [1, 0.1, 1]';

ODEsys.delta_U_lower = [ -inf, -inf, -inf]';
ODEsys.delta_U_upper = [ inf, inf, inf]';

%% Dilutions - change of concentrations because of streams in and out
%Sugars
D_Glu= F_SSL_in*(SSL_Glu_in-Glu)/V + F_RiboXyl_in*(RiboXyl_Glu_in-Glu)/V; 
D_Man=F_SSL_in*(SSL_Man_in-Man)/V + F_RiboXyl_in*(RiboXyl_Man_in-Man)/V;
D_Xyl=F_SSL_in*(SSL_Xyl_in-Xyl)/V + F_RiboXyl_in*(RiboXyl_Xyl_in-Xyl)/V;
%Biomass
D_RiboXyl= F_SSL_in*(0-X_RiboXyl)/V+ ...
    F_RiboXyl_in*(RiboXyl_X_RiboXyl_in-X_RiboXyl)/V;
%Product
D_P= F_SSL_in*(SSL_P_in-P)/V+ ...
     F_RiboXyl_in*(RiboXyl_P_in-P)/V ;
%%  ODEs
X_RiboXyl_dot = rx_RiboXyl + D_RiboXyl; 
Glu_dot = rs1_RiboXyl + D_Glu;
Man_dot = rs2_RiboXyl +  D_Man;
Xyl_dot= rs3_RiboXyl +  D_Xyl;
P_dot = rp_RiboXyl + D_P;
V_dot=F_RiboXyl_in +  F_SSL_in - F_out; 
V_RiboGlu_dot=-F_RiboXyl_in;
ODEsys.x_dot = [X_RiboXyl_dot, Glu_dot, Man_dot, Xyl_dot, P_dot, V_dot, V_RiboGlu_dot]'; % endre

end
