%
%     This file is part of CasADi.
%
%     CasADi -- A symbolic framework for dynamic optimization.
%     Copyright (C) 2010-2014 Joel Andersson, Joris Gillis, Moritz Diehl,
%                             K.U. Leuven. All rights reserved.
%     Copyright (C) 2011-2014 Greg Horn
%
%     CasADi is free software; you can redistribute it and/or
%     modify it under the terms of the GNU Lesser General Public
%     License as published by the Free Software Foundation; either
%     version 3 of the License, or (at your option) any later version.
%
%     CasADi is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%     Lesser General Public License for more details.
%
%     You should have received a copy of the GNU Lesser General Public
%     License along with CasADi; if not, write to the Free Software
%     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
%


% A function initializing the optimal control problem
% Based on an implementation of direct collocation by Joel Andersson

function [OCP, solver] = direct_collocation_func(ODEmodel, OCPprops, x0)

import casadi.*

% Degree of interpolating polynomial
intPolDeg = OCPprops.d;


% Get collocation points
tau_root = [0 collocation_points(intPolDeg, 'legendre')];

% Coefficients of the collocation equation
C = zeros(intPolDeg+1,intPolDeg+1);

% Coefficients of the continuity equation
D = zeros(intPolDeg+1, 1);

% Coefficients of the quadrature function
B = zeros(intPolDeg+1, 1);

% Construct polynomial basis
for j=1:intPolDeg+1
  % Construct Lagrange polynomials to get the polynomial basis at the collocation point
  coeff = 1;
  for r=1:intPolDeg+1
    if r ~= j
      coeff = conv(coeff, [1, -tau_root(r)]);
      coeff = coeff / (tau_root(j)-tau_root(r));
    end
  end
  % Evaluate the polynomial at the final time to get the coefficients of the continuity equation
  D(j) = polyval(coeff, 1.0);

  % Evaluate the time derivative of the polynomial at all collocation points to get the coefficients of the continuity equation
  pder = polyder(coeff);
  for r=1:intPolDeg+1
    C(j,r) = polyval(pder, tau_root(r));
  end

  % Evaluate the integral of the polynomial to get the coefficients of the quadrature function
  pint = polyint(coeff);
  B(j) = polyval(pint, 1.0);
end
% Time horizon
T = OCPprops.T;

% Declare model variables
x = ODEmodel.x;
u = ODEmodel.u;
d = ODEmodel.d;
d0 = ODEmodel.d0;
nx = length(x);
nu = length(u);

% Model equations

xdot = ODEmodel.x_dot;

% Substitute for nominal value of disturbance
xdot = substitute(xdot,d,d0);

% Objective term
L = OCPprops.stageCost;

% Continuous time dynamics
f = Function('f', {x, u}, {xdot, L});

% Control discretization
N = OCPprops.N;
h = T/N;

% Constraints
lbx = ODEmodel.x_lower;
ubx = ODEmodel.x_upper;

lbu = ODEmodel.u_lower;
ubu = ODEmodel.u_upper;


% Make x0 a column matrix
x0 = reshape(x0,nx,1);

% Start with an empty NLP
w={};
w0 = [];
lbw = [];
ubw = [];
J = 0;
g={};
lbg = [];
ubg = [];

% Initial conditions
%Equality constraint?
X0 = MX.sym('X0', nx);
w = {w{:}, X0};
lbw = [lbw; lbx];
ubw = [ubw; ubx];
w0 = [w0; x0];

Xk = X0;

% Variable initial conditions (probably an ineffective implementation, but
% extends easily to multistage)
%Inequality constraint?
X0_var = MX.sym('X0_var',nx);
            
g   = {g{:},X0 - X0_var};
lbg = [lbg;zeros(nx,1)];
ubg = [ubg;zeros(nx,1)];

% Formulate the NLP
for k=0:N-1
    
    % New NLP variable for the control
    Uk = MX.sym(['U_' num2str(k)], nu); % Creating input number k
    w = {w{:}, Uk};                 % Adding input number k as decision variable
    lbw = [lbw; lbu];                % Lower bounds of Uk
    ubw = [ubw;  ubu];                % Upper bounds of Uk
    w0 = [w0;  ubu];%zeros(nu,1)];                  % Initial value

    % State at collocation points
    Xkj = {};
    for j=1:intPolDeg   % For all collocation points d
        Xkj{j} = MX.sym(['X_' num2str(k) '_' num2str(j)], nx);   % Creating state number j interpolating between k and k+1: Two variables for two states
        w = {w{:}, Xkj{j}};                                     % Adding as decision variable
        lbw = [lbw; lbx];                               % Lower bound
        ubw = [ubw;  ubx];                                % Upper bound
        w0 = [w0; x0];                                        % Initial value
    end

    % Loop over collocation points
    Xk_end = D(1)*Xk;
    
    for j=1:intPolDeg
       % Expression for the state derivative at the collocation point
       xp = C(1,j+1)*Xk;
       for r=1:intPolDeg
           xp = xp + C(r+1,j+1)*Xkj{r};
       end

       % Append collocation equations
       [fj, qj] = f(Xkj{j},Uk);
       g = {g{:}, h*fj - xp};
       lbg = [lbg; zeros(nx,1)];
       ubg = [ubg; zeros(nx,1)];

       % Add contribution to the end state
       Xk_end = Xk_end + D(j+1)*Xkj{j};

       % Add contribution to quadrature function
       J = J + B(j+1)*qj*h;
    end

    % New NLP variable for state at end of interval
    Xk = MX.sym(['X_' num2str(k+1)], nx);
    w = {w{:}, Xk};
    lbw = [lbw; lbx];
    ubw = [ubw;  ubx];
    w0 = [w0; x0];

    % Add equality constraint
    g = {g{:}, Xk_end-Xk};
    lbg = [lbg; zeros(nx,1)];
    ubg = [ubg; zeros(nx,1)];
end

OCP = struct;
OCP.J = J;
OCP.w = w;
OCP.g = g;
OCP.w0 = w0;
OCP.lbw = lbw;
OCP.ubw = ubw;
OCP.lbg = lbg;
OCP.ubg = ubg;

% Calculating corresponding times to each collocation point
repColPoints = [0, repmat([tau_root(2:end) 1]*h,1,N)]';
baseTimes = [0; reshape((0:h:T-h).*ones(1+intPolDeg,1),[N*(intPolDeg+1),1])];

OCP.timeseries = repColPoints+baseTimes;    % Timeseries

opts = struct('warn_initial_bounds',false, ...
    'print_time',false, ...
    'eval_errors_fatal',true, ...
    'ipopt',struct('print_level',1) ...
    );


OCP_NLP = struct('f', J, 'x', vertcat(w{:}), 'g', vertcat(g{:}), 'p', X0_var);
solver = nlpsol('solver', 'ipopt', OCP_NLP);


end